@@ -1,7 +1,10 @@
 #
 # By Krister Hedfors
 #
-
+# + pack/unpack everywhere
+# + db-sorted iteritems
+#
+#
 __all__ = ['sqldict']
 
 __doc__ = '''
@@ -56,6 +59,7 @@ class BaseSQLDict(DictMixin):
                             ns='',
                             create=0,
                             flags=_flags,
+                            sort=1, # 0=dont, 1=keys, 2=values
                             keysize=_KEYSIZE,
                             valsize=_VALSIZE,
                             nssize=_NSSIZE,
@@ -69,6 +73,7 @@ class BaseSQLDict(DictMixin):
         self._tablename = tablename
         self._ns = ns
         self._create = create
+        self._sort = sort
         self._keysize = keysize
         self._valsize =  valsize
         self._nssize =  nssize
@@ -120,26 +125,144 @@ class BaseSQLDict(DictMixin):
         query = query.values(sqldict_val=val, sqldict_flags=flags)
         conn.execute(query)
 
+    def drop(self):
+        self._table.drop()
+
+    def pack_key(self, key, flags):
+        key_type = type(key)
+        if key_type is str:
+            pass
+        elif key_type is unicode:
+            flags |= self._F_UNICODE_KEY
+        else:
+            flags |= self._F_SERIALIZED_KEY
+            key = self._serialize(key)
+        return (key, flags)
+
+    def unpack_key(self, key, flags):
+        if flags & self._F_SERIALIZED_KEY:
+            key = self._deserialize(key)
+        elif type(key) is unicode and not (flags & self._F_UNICODE_KEY):
+            key = str(key)
+        return key
+
+
+    def pack_val(self, val, flags):
+        val_type = type(val)
+        if val_type is str:
+            pass
+        elif val_type is unicode:
+            flags |= self._F_UNICODE_VAL
+        else:
+            flags |= self._F_SERIALIZED_VAL
+            val = self._serialize(val)
+        return (val, flags)
+
+    def unpack_val(self, val, flags):
+        if flags & self._F_SERIALIZED_VAL:
+            val = self._deserialize(val)
+        elif type(val) is unicode and not (flags & self._F_UNICODE_VAL):
+            val = str(val)
+        return val
+
     #
     # dict methods
     # 
+    def iteritems(self):
+        conn = self._conn
+        ns = self._ns
+        table = self._table
+        sort = self._sort
+        #
+        #print 'keys'
+        query = select([table.c.sqldict_key,
+                        table.c.sqldict_val,
+                        table.c.sqldict_flags])
+        query = query.where(table.c.sqldict_ns == ns)
+        if sort == 1:
+            #print 'sort 1'
+            query = query.order_by(table.c.sqldict_key)
+        if sort == 2:
+            #print 'sort 2'
+            query = query.order_by(table.c.sqldict_val)
+        #print query
+        result = conn.execute(query)
+        for row in result:
+            key = row['sqldict_key']
+            val = row['sqldict_val']
+            flags = row['sqldict_flags']
+            key = self.unpack_key(key, flags)
+            val = self.unpack_val(val, flags)
+            yield (key, val)
+
+    def namespaces(self):
+        conn = self._conn
+        ns = self._ns
+        table = self._table
+        sort = self._sort
+        #
+        query = select([ distinct(table.c.sqldict_ns) ])
+        if sort == 1:
+            query = query.order_by(table.c.sqldict_ns)
+        if sort == 2:
+            query = query.order_by(table.c.sqldict_ns)
+        #print query
+        result = conn.execute(query)
+        namespaces = []
+        for row in result:
+            ns = row['sqldict_ns']
+            namespaces.append(ns)
+        return namespaces
+
+    def _dup_kwargs(self):
+        kwargs = dict(
+            conn=self._conn,
+            tablename=self._tablename,
+            ns=self._ns,
+            create=0,
+            sort=self._sort,
+            keysize=self._keysize,
+            valsize=self._valsize,
+            nssize=self._nssize,
+        )
+        return kwargs
+
+    def dup(self):
+        cls = self.__class__
+        kwargs = self._dup_kwargs()
+        duplicate = cls(**kwargs)
+        return duplicate
+
+    def using_ns(self, ns):
+        cls = self.__class__
+        kwargs = self._dup_kwargs()
+        kwargs['ns'] = ns
+        duplicate = cls(**kwargs)
+        return duplicate
+
+        
     def keys(self):
         conn = self._conn
         ns = self._ns
         table = self._table
+        sort = self._sort
         #
         #print 'keys'
         query = select([table.c.sqldict_key, table.c.sqldict_flags])
         query = query.where(table.c.sqldict_ns == ns)
+        if sort == 1:
+            #print 'sort 1'
+            query = query.order_by(table.c.sqldict_key)
+        if sort == 2:
+            #print 'sort 2'
+            query = query.order_by(table.c.sqldict_val)
+        #print query
         result = conn.execute(query)
         keys = []
         for row in result:
             key = row['sqldict_key']
             flags = row['sqldict_flags']
-            if flags & self._F_SERIALIZED_KEY:
-                key = self._deserialize(key)
-            elif type(key) is unicode and not (flags & self._F_UNICODE_KEY):
-                key = str(key)
+            key = self.unpack_key(key, flags)
             keys.append(key)
         return keys
 
@@ -158,11 +281,7 @@ class BaseSQLDict(DictMixin):
         for row in result:
             val = row['sqldict_val']
             flags = row['sqldict_flags']
-            if flags & self._F_SERIALIZED_VAL:
-                val = self._deserialize(val)
-            else:
-                if type(val) is unicode and not (flags & self._F_UNICODE_VAL):
-                    val = str(val)
+            val = self.unpack_val(val, flags)
             return val
         raise KeyError
 
@@ -190,17 +309,8 @@ class BaseSQLDict(DictMixin):
         flags = self._flags
         ns = self._ns
 
-        if not type(key) in (str, unicode):
-            key = self._serialize(key)
-            flags |= self._F_SERIALIZED_KEY
-        if not type(val) in (str, unicode):
-            val = self._serialize(val)
-            flags |= self._F_SERIALIZED_VAL
-
-        if type(key) is unicode:
-            flags |= self._F_UNICODE_KEY
-        if type(val) is unicode:
-            flags |= self._F_UNICODE_VAL
+        key, flags = self.pack_key(key, 0)
+        val, flags = self.pack_val(val, flags)
 
         conn = self._conn
         table = self._table
@@ -219,8 +329,10 @@ class BaseSQLDict(DictMixin):
 
 
 
-def sqldict(dburi='sqlite://', engine=None, conn=None, tablename='',
+def sqldict(dburi='sqlite://', engine=None, conn=None, ns='', tablename='',
                 create=0, **kw):
+    '''
+    '''
 
     if not (dburi or engine or conn):
         error = 'sqldict(): must specify exactly one of `dburi`, `engine` or `conn`'
@@ -239,7 +351,7 @@ def sqldict(dburi='sqlite://', engine=None, conn=None, tablename='',
     if engine:
         conn = engine.connect()
 
-    return BaseSQLDict(conn=conn, tablename=tablename, create=create, **kw)
+    return BaseSQLDict(conn=conn, ns=ns, tablename=tablename, create=create, **kw)
 
 
 
